@@ -70,9 +70,9 @@ class SparkConnection:
         self.spark_session = spark_session
 
     def close(self):
-        if self.spark_conext is not None:
-            self.spark_conext.stop()  # probably only for local demos
-            self.spark_conext = None
+        if self.spark_context is not None:
+            self.spark_context.stop()  # probably only for local demos
+            self.spark_context = None
         if self.spark_session is not None:
             self.spark_session = None
 
@@ -101,12 +101,7 @@ class SparkSQLModel(data_algebra.db_model.DBModel):
     def read_query(self, conn, q):
         assert isinstance(conn, SparkConnection)
         if isinstance(q, data_algebra.data_ops.ViewRepresentation):
-            temp_tables = dict()
-            q = q.to_sql(db_model=self, temp_tables=temp_tables)
-            if len(temp_tables) > 1:
-                raise ValueError(
-                    "ops require management of temp tables, please collect them via to_sql(temp_tables)"
-                )
+            q = q.to_sql(db_model=self)
         else:
             q = str(q)
         res = conn.spark_session.sql(q)
