@@ -2,7 +2,7 @@ from abc import ABC
 from typing import Set, Dict, List, Union
 import numbers
 import re
-import collections
+import typing
 
 import data_algebra
 import data_algebra.expr_parse
@@ -29,7 +29,7 @@ except ImportError:
 
 
 # noinspection PyBroadException
-def pretty_format_python(python_str, *, black_mode=None):
+def pretty_format_python(python_str: str, *, black_mode=None) -> str:
     assert isinstance(python_str, str)
     formatted_python = python_str
     if _have_black:
@@ -48,11 +48,19 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     column_names: List[str]
     column_set: data_algebra.OrderedSet.OrderedSet
+    column_types: typing.Optional[Dict[str, type]]
     sources: List[
         "ViewRepresentation"
     ]  # https://www.python.org/dev/peps/pep-0484/#forward-references
 
-    def __init__(self, column_names, *, sources=None, node_name):
+    def __init__(
+        self,
+        column_names: List[str],
+        *,
+        column_types: typing.Optional[Dict[str, type]] = None,
+        sources: List["ViewRepresentation"] = None,
+        node_name: str,
+    ):
         if isinstance(column_names, str):
             column_names = [column_names]
         self.column_names = [c for c in column_names]
@@ -74,11 +82,14 @@ class ViewRepresentation(OperatorPlatform, ABC):
         for si in sources:
             assert isinstance(si, ViewRepresentation)
         self.sources = [si for si in sources]
+        self.column_types = None
+        if column_types is not None:
+            self.column_types = column_types.copy()
         OperatorPlatform.__init__(
             self, node_name=node_name, column_map=collections.OrderedDict(**column_dict)
         )
 
-    def merged_rep_id(self):
+    def merged_rep_id(self) -> str:
         return "ops+ " + str(id(self))
 
     # convenience
@@ -94,6 +105,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         """
         tables = self.get_tables()
         for t in tables.values():
+            assert isinstance(t, TableDescription)
             assert t.head is not None
             if len(tables) > 1:
                 assert t.table_name_was_set_by_user
@@ -101,7 +113,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
                 assert t.limit_was is None
                 assert t.nrows == t.head.shape[0]
         data_map = {k: v.head for k, v in tables.items()}
-        return self.eval(data_map = data_map, data_model=data_model, narrow=narrow)
+        return self.eval(data_map=data_map, data_model=data_model, narrow=narrow)
 
     # characterization
 
@@ -113,7 +125,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
             s = self.sources[i]
             ti = s.get_tables()
             for (k, v) in ti.items():
-                assert isinstance(v, OperatorPlatform)
+                assert isinstance(v, TableDescription)
                 assert v.node_name == "TableDescription"
                 if k in tables.keys():
                     if not v.same_table(tables[k]):
@@ -125,20 +137,20 @@ class ViewRepresentation(OperatorPlatform, ABC):
         return tables
 
     def columns_used_from_sources(self, using=None):
-        """Get column names used from direct source nodes when this ops is exececuted
+        """Get column names used from direct source nodes when this ops is executed
         with the using columns (None means all)."""
         raise NotImplementedError("base method called")
 
     def columns_produced(self):
         return self.column_names.copy()
 
-    def _columns_used_implementation(self, *, using, columns_currenty_using_records):
+    def _columns_used_implementation(self, *, using, columns_currently_using_records):
         self_merged_rep_id = self.merged_rep_id()
         try:
-            crec = columns_currenty_using_records[self_merged_rep_id]
+            crec = columns_currently_using_records[self_merged_rep_id]
         except KeyError:
             crec = set()
-            columns_currenty_using_records[self_merged_rep_id] = crec
+            columns_currently_using_records[self_merged_rep_id] = crec
         if using is None:
             crec.update(self.column_names)
         else:
@@ -150,23 +162,23 @@ class ViewRepresentation(OperatorPlatform, ABC):
         for i in range(len(self.sources)):
             self.sources[i]._columns_used_implementation(
                 using=cu_list[i],
-                columns_currenty_using_records=columns_currenty_using_records,
+                columns_currently_using_records=columns_currently_using_records,
             )
 
     def columns_used(self, *, using=None):
         """Determine which columns are used from source tables."""
 
         tables = self.get_tables()
-        columns_currenty_using_records = {
+        columns_currently_using_records = {
             v.merged_rep_id(): set() for v in tables.values()
         }
         self._columns_used_implementation(
-            using=using, columns_currenty_using_records=columns_currenty_using_records
+            using=using, columns_currently_using_records=columns_currently_using_records
         )
         columns_used = dict()
         for k in tables.keys():
             ti = tables[k]
-            vi = columns_currenty_using_records[ti.merged_rep_id()]
+            vi = columns_currently_using_records[ti.merged_rep_id()]
             columns_used[k] = vi.copy()
         return columns_used
 
@@ -198,7 +210,9 @@ class ViewRepresentation(OperatorPlatform, ABC):
             strict = True
         python_str = (
             "(\n"
-            + self.to_python_implementation(indent=indent, strict=strict, print_sources=True)
+            + self.to_python_implementation(
+                indent=indent, strict=strict, print_sources=True
+            )
             + "\n)\n"
         )
         if pretty:
@@ -243,26 +257,20 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     # query generation
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         raise NotImplementedError("base method called")
 
     def to_sql(
-        self,
-        db_model,
-        *,
-        temp_tables=None,
-        sql_format_options=None,
+        self, db_model, *, sql_format_options=None,
     ):
         if isinstance(db_model, data_algebra.db_model.DBHandle):
             db_model = db_model.db_model
         assert isinstance(db_model, data_algebra.db_model.DBModel)
         if sql_format_options is None:
             sql_format_options = db_model.default_SQL_format_options
-        return db_model.to_sql(
-            ops=self,
-            temp_tables=temp_tables,
-            sql_format_options=sql_format_options,
-        )
+        return db_model.to_sql(ops=self, sql_format_options=sql_format_options,)
 
     # Pandas realization
 
@@ -273,7 +281,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         """
         Check tables supplied meet data consistency constraints.
 
-        data_model: dictionairy of column name lists.
+        data_model: dictionary of column name lists.
         """
         self.columns_used()  # for table consistency check/raise
         forbidden = self.forbidden_columns()
@@ -331,7 +339,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
     # noinspection PyPep8Naming
     def transform(self, X, *, data_model=None, narrow=True):
         """
-        Apply data transfrom to a table
+        Apply data transform to a table
 
         :param X: tale to apply to
         :param data_model: data model for Pandas execution
@@ -469,6 +477,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         :param b: right table
         :param by: list of keys to join by
         :param jointype: one of 'INNER', 'LEFT', 'RIGHT', 'OUTER', 'FULL', 'CROSS' (case insensitive)
+        :param check_all_common_keys_in_by: logical if True don't allow shared column names that are not in by-term
         :return: ops describing join
         """
         assert isinstance(b, ViewRepresentation)
@@ -479,7 +488,8 @@ class ViewRepresentation(OperatorPlatform, ABC):
             b=b,
             by=by,
             jointype=jointype,
-            check_all_common_keys_in_by=check_all_common_keys_in_by)
+            check_all_common_keys_in_by=check_all_common_keys_in_by,
+        )
 
     def concat_rows(self, b, *, id_column="source_name", a_name="a", b_name="b"):
         if b is None:
@@ -588,13 +598,13 @@ class TableDescription(ViewRepresentation):
         column_names,
         qualifiers=None,
         sql_meta=None,
-        column_types=None,
+        column_types: typing.Optional[Dict[str, type]] = None,
         head=None,
         limit_was=None,
-        nrows=None
+        nrows=None,
     ):
         ViewRepresentation.__init__(
-            self, column_names=column_names, node_name="TableDescription"
+            self, column_names=column_names, node_name="TableDescription", column_types=column_types,
         )
         if table_name is None:
             self.table_name_was_set_by_user = False
@@ -613,9 +623,6 @@ class TableDescription(ViewRepresentation):
         if isinstance(column_names, str):
             column_names = [column_names]
         self.column_names = [c for c in column_names]
-        self.column_types = None
-        if column_types is not None:
-            self.column_types = column_types.copy()
         if qualifiers is None:
             qualifiers = {}
         assert isinstance(qualifiers, dict)
@@ -720,9 +727,14 @@ class TableDescription(ViewRepresentation):
     def columns_used_from_sources(self, using=None):
         return []  # no inputs to table description
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.table_def_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def __str__(self):
@@ -753,21 +765,21 @@ def describe_table(
     qualifiers=None,
     sql_meta=None,
     column_types=None,
-    row_limit=7,
+    row_limit: typing.Optional[int] = 7,
     keep_sample=True,
-    keep_all=False
-):
+    keep_all=False,
+) -> TableDescription:
     """
     :param d: pandas table to describe
     :param table_name: name of table
-    :param qualifiers: optional able qualifiers
-    :param sql_meta: optional sql meta information map
+    :param qualifiers: optional, able qualifiers
+    :param sql_meta: optional, sql meta information map
+    :param column_types: optional, map of column types
     :param row_limit: how many rows to sample
     :param keep_sample: logical, if True retain head of table
     :param keep_all: logical, if True retain all of table
     :return: TableDescription
     """
-    assert d is not None
     assert not isinstance(d, OperatorPlatform)
     assert not isinstance(d, ViewRepresentation)
     column_names = [c for c in d.columns]
@@ -797,7 +809,7 @@ def describe_table(
     )
 
 
-def table(d, table_name):
+def table(d, table_name=None):
     """
     Capture a table for later use
 
@@ -975,7 +987,9 @@ class ExtendNode(ViewRepresentation):
                     if len(opk.args) > 1:
                         for i in range(1, len(opk.args)):
                             if not isinstance(opk.args[i], data_algebra.expr_rep.Value):
-                                raise ValueError("window function with more than one non-value argument")
+                                raise ValueError(
+                                    "window function with more than one non-value argument"
+                                )
                     for i in range(len(opk.args)):
                         if not (
                             isinstance(
@@ -1032,9 +1046,15 @@ class ExtendNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
-        return db_model.extend_to_sql(self, using=using, temp_id_source=temp_id_source,
-                                      sql_format_options=sql_format_options)
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
+        return db_model.extend_to_sql(
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
+        )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
@@ -1097,10 +1117,7 @@ class ProjectNode(ViewRepresentation):
                     in data_algebra.expr_rep.fn_names_that_imply_ordered_windowed_situation
                 ):
                     raise ValueError(str(opk) + "' is not allowed in project")
-                if (
-                    opk.op
-                    in data_algebra.expr_rep.fn_names_not_allowed_in_project
-                ):
+                if opk.op in data_algebra.expr_rep.fn_names_not_allowed_in_project:
                     raise ValueError(str(opk) + "' is not allowed in project")
             else:
                 raise ValueError(
@@ -1173,9 +1190,15 @@ class ProjectNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
-        return db_model.project_to_sql(self, using=using, temp_id_source=temp_id_source,
-                                       sql_format_options=sql_format_options)
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
+        return db_model.project_to_sql(
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
+        )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
@@ -1237,9 +1260,14 @@ class SelectRowsNode(ViewRepresentation):
         s = s + ("select_rows(" + self.expr.to_python().__repr__() + ")")
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.select_rows_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1306,9 +1334,14 @@ class SelectColumnsNode(ViewRepresentation):
         s = s + ("select_columns(" + self.column_selection.__repr__() + ")")
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.select_columns_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1375,9 +1408,14 @@ class DropColumnsNode(ViewRepresentation):
         s = s + ("drop_columns(" + self.column_deletions.__repr__() + ")")
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.drop_columns_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1456,9 +1494,15 @@ class OrderRowsNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
-        return db_model.order_to_sql(self, using=using, temp_id_source=temp_id_source,
-                                     sql_format_options=sql_format_options)
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
+        return db_model.order_to_sql(
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
+        )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
@@ -1550,9 +1594,15 @@ class RenameColumnsNode(ViewRepresentation):
         s = s + ("rename_columns(" + self.column_remapping.__repr__() + ")")
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
-        return db_model.rename_to_sql(self, using=using, temp_id_source=temp_id_source,
-                                      sql_format_options=sql_format_options)
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
+        return db_model.rename_to_sql(
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
+        )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
@@ -1600,7 +1650,8 @@ class NaturalJoinNode(ViewRepresentation):
             if len(missing_common) > 0:
                 raise KeyError(
                     "check_all_common_keys_in_by set, and the following common keys are are not in the by-clause: "
-                    + str(missing_common))
+                    + str(missing_common)
+                )
         ViewRepresentation.__init__(
             self,
             column_names=column_names,
@@ -1660,9 +1711,14 @@ class NaturalJoinNode(ViewRepresentation):
         )
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.natural_join_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1676,6 +1732,8 @@ class ConcatRowsNode(ViewRepresentation):
 
     def __init__(self, a, b, *, id_column="table_name", a_name="a", b_name="b"):
         # check set of tables is consistent in both sub-dags
+        assert isinstance(a, ViewRepresentation)
+        assert isinstance(b, ViewRepresentation)
         a_tables = a.get_tables()
         b_tables = b.get_tables()
         common_keys = set(a_tables.keys()).intersection(b_tables.keys())
@@ -1691,10 +1749,19 @@ class ConcatRowsNode(ViewRepresentation):
         if id_column is not None and id_column in sources[0].column_names:
             raise ValueError("id_column should not be an input table column name")
         column_names = sources[0].column_names.copy()
-        if (isinstance(a, TableDescription) and (a.column_types is not None) and (len(a.column_types) > 0)
-            and isinstance(b, TableDescription) and (b.column_types is not None) and (len(b.column_types) > 0)):
+        if (
+            isinstance(a, TableDescription)
+            and (a.column_types is not None)
+            and (len(a.column_types) > 0)
+            and isinstance(b, TableDescription)
+            and (b.column_types is not None)
+            and (len(b.column_types) > 0)
+        ):
             for c in column_names:
-                assert len({a.column_types[c]}.union({b.column_types[c]}) - {type(None)}) <= 1
+                assert (
+                    len({a.column_types[c]}.union({b.column_types[c]}) - {type(None)})
+                    <= 1
+                )
         if id_column is not None:
             assert id_column not in column_names
             column_names.append(id_column)
@@ -1763,9 +1830,14 @@ class ConcatRowsNode(ViewRepresentation):
         )
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         return db_model.concat_rows_to_sql(
-            self, using=using, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            self,
+            using=using,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
 
     def eval_implementation(self, *, data_map, data_model, narrow):
@@ -1789,18 +1861,6 @@ class ConvertRecordsNode(ViewRepresentation):
             node_name="ConvertRecordsNode",
         )
 
-    def control_out_table(self, *, temp_id_source):
-        if self.temp_namer is None:
-            view_name = "cdata_temp_record_" + str(temp_id_source[0])
-        else:
-            view_name = self.temp_namer(temp_id_source[0])
-        temp_id_source[0] = temp_id_source[0] + 1
-        res = TableDescription(
-            table_name=view_name,
-            column_names=self.record_map.blocks_out.control_table.columns,
-        )
-        return res
-
     def apply_to(self, a, *, target_table_key=None):
         new_sources = [
             s.apply_to(a, target_table_key=target_table_key) for s in self.sources
@@ -1831,12 +1891,17 @@ class ConvertRecordsNode(ViewRepresentation):
         s = s + ")"
         return s
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source, sql_format_options=None):
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ):
         if temp_id_source is None:
             temp_id_source = [0]
         # TODO: narrow to what we are using
         sub_query = self.sources[0].to_near_sql_implementation(
-            db_model=db_model, using=None, temp_id_source=temp_id_source, sql_format_options=sql_format_options
+            db_model=db_model,
+            using=None,
+            temp_id_source=temp_id_source,
+            sql_format_options=sql_format_options,
         )
         # claims to use all columns
         query = sub_query.to_sql(
@@ -1844,39 +1909,24 @@ class ConvertRecordsNode(ViewRepresentation):
             db_model=db_model,
             force_sql=True,
         )
-        control_out_table = None
         if self.record_map.blocks_in is not None:
             query = db_model.blocks_to_row_recs_query(
                 query, record_spec=self.record_map.blocks_in
             )
         if self.record_map.blocks_out is not None:
-            control_out_table = self.control_out_table(temp_id_source=temp_id_source)
             query = db_model.row_recs_to_blocks_query(
-                query,
-                record_spec=self.record_map.blocks_out,
-                control_view=control_out_table,
+                query, record_spec=self.record_map.blocks_out,
             )
         view_name = "convert_records_in_" + str(temp_id_source[0])
         temp_id_source[0] = temp_id_source[0] + 1
         prev_view_name = "convert_records_out_" + str(temp_id_source[0])
         temp_id_source[0] = temp_id_source[0] + 1
         terms = {k: None for k in self.record_map.columns_produced}
-        temp_tables = sub_query.temp_tables.copy()
-        if control_out_table is not None:
-            if control_out_table.key in temp_tables.keys():
-                raise ValueError(
-                    "key collision in temp_tables construction: "
-                    + control_out_table.key
-                )
-            temp_tables[
-                control_out_table.key
-            ] = self.record_map.blocks_out.control_table
         near_sql = data_algebra.near_sql.NearSQLq(
             quoted_query_name=db_model.quote_identifier(view_name),
             prev_quoted_query_name=db_model.quote_identifier(prev_view_name),
             query=query,
             terms=terms,
-            temp_tables=temp_tables,
             annotation="convert records",
         )
         return near_sql
