@@ -23,6 +23,17 @@ class OperatorPlatform:
         self.node_name = node_name
         self.column_map = column_map.copy()
 
+    def eval(self, data_map, *, data_model=None, narrow=True):
+        """
+        Evaluate operators with respect to Pandas data frames.
+
+        :param data_map: map from table names to data frames
+        :param data_model: adaptor to data dialect (Pandas for now)
+        :param narrow: logical, if True don't copy unexpected columns
+        :return: table result
+        """
+        raise NotImplementedError("base class called")
+
     # noinspection PyPep8Naming
     def transform(self, X, *, data_model=None, narrow=True):
         """
@@ -95,6 +106,26 @@ class OperatorPlatform:
         """
         return other.apply_to(self)
 
+    # convenience
+
+    def ex(self, *, data_model=None, narrow=True, allow_limited_tables=False):
+        """
+        Evaluate operators with respect to Pandas data frames already stored in the operator chain.
+
+        :param data_model: adaptor to data dialect (Pandas for now)
+        :param narrow: logical, if True don't copy unexpected columns
+        :param allow_limited_tables: logical, if True allow execution on non-complete tables
+        :return: table result
+        """
+        raise NotImplementedError("base class called")
+
+    # characterization
+
+    def get_tables(self):
+        """Get a dictionary of all tables used in an operator DAG,
+        raise an exception if the values are not consistent."""
+        raise NotImplementedError("base class called")
+
     # info
 
     def columns_produced(self):
@@ -105,7 +136,7 @@ class OperatorPlatform:
     def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
         raise NotImplementedError("base method called")
 
-    # define builders for all non-initial node types on base class
+    # define builders for all non-initial ops types on base class
 
     def extend_parsed(
         self, parsed_ops, *, partition_by=None, order_by=None, reverse=None
@@ -121,7 +152,7 @@ class OperatorPlatform:
     def project(self, ops=None, *, group_by=None):
         raise NotImplementedError("base class called")
 
-    def natural_join(self, b, *, by, jointype):
+    def natural_join(self, b, *, by, jointype, check_all_common_keys_in_by=False):
         raise NotImplementedError("base class called")
 
     def concat_rows(self, b, *, id_column="source_name", a_name="a", b_name="b"):
@@ -152,7 +183,7 @@ class OperatorPlatform:
         self, blocks_in=None, blocks_out=None, strict=False, temp_namer=None
     ):
         if (blocks_in is None) and (blocks_out is None):
-            return self  # NO-OP, return source node
+            return self  # NO-OP, return source ops
         return self.convert_records(
             data_algebra.cdata.RecordMap(
                 blocks_in=blocks_in, blocks_out=blocks_out, strict=strict
