@@ -13,12 +13,9 @@ class NearSQL(ABC):
     Represent SQL queries in a mostly string-form
     """
 
-    def __init__(
-        self, *, terms, quoted_query_name, temp_tables, is_table=False, annotation=None
-    ):
+    def __init__(self, *, terms, quoted_query_name, is_table=False, annotation=None):
         assert isinstance(terms, (dict, type(None)))
         assert isinstance(quoted_query_name, str)
-        assert isinstance(temp_tables, dict)
         assert isinstance(is_table, bool)
         assert isinstance(annotation, (str, type(None)))
         self.terms = None
@@ -28,7 +25,6 @@ class NearSQL(ABC):
                 self.terms = terms.copy()
         self.quoted_query_name = quoted_query_name
         self.is_table = is_table
-        self.temp_tables = temp_tables.copy()
         self.annotation = annotation
 
     def to_bound_near_sql(self, *, columns=None, force_sql=False, constants=None):
@@ -37,7 +33,13 @@ class NearSQL(ABC):
         )
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         raise NotImplementedError("base method called")
 
@@ -121,15 +123,17 @@ class NearSQLContainer:
 class NearSQLCommonTableExpression(NearSQL):
     def __init__(self, *, quoted_query_name):
         NearSQL.__init__(
-            self,
-            terms=None,
-            quoted_query_name=quoted_query_name,
-            temp_tables=dict(),
-            is_table=True,
+            self, terms=None, quoted_query_name=quoted_query_name, is_table=True,
         )
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         return db_model.nearsqlcte_to_sql_(
             near_sql=self,
@@ -143,16 +147,18 @@ class NearSQLCommonTableExpression(NearSQL):
 class NearSQLTable(NearSQL):
     def __init__(self, *, terms, quoted_query_name, quoted_table_name):
         NearSQL.__init__(
-            self,
-            terms=terms,
-            quoted_query_name=quoted_query_name,
-            temp_tables=dict(),
-            is_table=True,
+            self, terms=terms, quoted_query_name=quoted_query_name, is_table=True,
         )
         self.quoted_table_name = quoted_table_name
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         return db_model.nearsqltable_to_sql_(
             near_sql=self,
@@ -171,7 +177,6 @@ class NearSQLUnaryStep(NearSQL):
         quoted_query_name,
         sub_sql,
         suffix=None,
-        temp_tables,
         annotation=None,
         mergeable=False,
         declared_term_dependencies=None,
@@ -185,7 +190,6 @@ class NearSQLUnaryStep(NearSQL):
             self,
             terms=terms,
             quoted_query_name=quoted_query_name,
-            temp_tables=temp_tables,
             annotation=annotation,
         )
         self.sub_sql = sub_sql
@@ -199,7 +203,13 @@ class NearSQLUnaryStep(NearSQL):
             self.mergeable = False
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         return db_model.nearsqlunary_to_sql_(
             near_sql=self,
@@ -221,7 +231,6 @@ class NearSQLUnaryStep(NearSQL):
             quoted_query_name=self.quoted_query_name,
             sub_sql=stub,
             suffix=self.suffix,
-            temp_tables=self.temp_tables,
             annotation=self.annotation,
         )
         sequence.append(stubbed_step)
@@ -238,8 +247,7 @@ class NearSQLBinaryStep(NearSQL):
         joiner,
         sub_sql2,
         suffix=None,
-        temp_tables,
-        annotation=None
+        annotation=None,
     ):
         assert isinstance(sub_sql1, NearSQLContainer)
         assert isinstance(sub_sql2, NearSQLContainer)
@@ -251,7 +259,6 @@ class NearSQLBinaryStep(NearSQL):
             self,
             terms=terms,
             quoted_query_name=quoted_query_name,
-            temp_tables=temp_tables,
             annotation=annotation,
         )
         self.sub_sql1 = sub_sql1
@@ -260,7 +267,13 @@ class NearSQLBinaryStep(NearSQL):
         self.suffix = suffix
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         return db_model.nearsqlbinary_to_sql_(
             near_sql=self,
@@ -299,7 +312,6 @@ class NearSQLBinaryStep(NearSQL):
             joiner=self.joiner,
             sub_sql2=stub2,
             suffix=self.suffix,
-            temp_tables=self.temp_tables,
             annotation=self.annotation,
         )
         sequence.append(stubbed_step)
@@ -319,8 +331,7 @@ class NearSQLq(NearSQL):
         query,
         terms,
         prev_quoted_query_name,
-        temp_tables,
-        annotation=None
+        annotation=None,
     ):
         assert isinstance(query, str)
         assert isinstance(prev_quoted_query_name, str)
@@ -328,14 +339,19 @@ class NearSQLq(NearSQL):
             self,
             terms=terms,
             quoted_query_name=quoted_query_name,
-            temp_tables=temp_tables,
             annotation=annotation,
         )
         self.query = query
         self.prev_quoted_query_name = prev_quoted_query_name
 
     def to_sql(
-        self, *, columns=None, force_sql=False, constants=None, db_model, sql_format_options=None
+        self,
+        *,
+        columns=None,
+        force_sql=False,
+        constants=None,
+        db_model,
+        sql_format_options=None,
     ):
         return db_model.nearsqlq_to_sql_(
             near_sql=self,
