@@ -82,7 +82,7 @@ def equivalent_frames(
     if set(a_columns) != set(b_columns):
         return False
     if check_column_order:
-        if not all([a.columns[i] == b.columns[i] for i in range(a.shape[0])]):
+        if not numpy.all([a.columns[i] == b.columns[i] for i in range(a.shape[0])]):
             return False
     else:
         # re-order b into a's column order
@@ -112,9 +112,9 @@ def equivalent_frames(
             cb_null = cb.isnull()
             if (ca_null is None) != (cb_null is None):
                 return False
-            if not all(ca_null == cb_null):
+            if not numpy.all(ca_null == cb_null):
                 return False
-            if not all(ca_null):
+            if not numpy.all(ca_null):
                 ca = ca[ca_null == False]
                 cb = cb[cb_null == False]
                 if local_data_model.can_convert_col_to_numeric(a[c]):
@@ -124,7 +124,7 @@ def equivalent_frames(
                     if numpy.max(dif) > float_tol:
                         return False
                 else:
-                    if not all(ca == cb):
+                    if not numpy.all(ca == cb):
                         return False
     return True
 
@@ -222,21 +222,16 @@ def check_transform_on_handles(
     if db_handles is not None:
         for db_handle in db_handles:
             to_del = set()
-            temp_tables = None
             sql_statements = []
             for initial_commas in [True, False]:
                 for use_with in [True, False]:
                     sql_format_options = data_algebra.db_model.SQLFormatOptions(
                         use_with=use_with,
                         annotate=True,
-                        sql_indent=' ',
-                        initial_commas=initial_commas)
-                    temp_tables = dict()
-                    sql = db_handle.to_sql(
-                        ops,
-                        temp_tables=temp_tables,
-                        sql_format_options=sql_format_options,
+                        sql_indent=" ",
+                        initial_commas=initial_commas,
                     )
+                    sql = db_handle.to_sql(ops, sql_format_options=sql_format_options,)
                     assert isinstance(sql, str)
                     sql_statements.append(sql)
                     # print(sql)
@@ -244,19 +239,14 @@ def check_transform_on_handles(
                 for (k, v) in data.items():
                     db_handle.insert_table(v, table_name=k, allow_overwrite=True)
                     to_del.add(k)
-                for (k, v) in temp_tables.items():
-                    db_handle.insert_table(v, table_name=k, allow_overwrite=True)
-                    to_del.add(k)
                 caught = None
                 res_db_sql = []
                 res_db_ops = None
-                run_ops_version = len(temp_tables) <= 0
                 try:
                     for sql in sql_statements:
                         res_db_sql_i = db_handle.read_query(sql)
                         res_db_sql.append(res_db_sql_i)
-                    if run_ops_version:
-                        res_db_ops = db_handle.read_query(ops)
+                    res_db_ops = db_handle.read_query(ops)
                 except Exception as e:
                     caught = e
                 for k in to_del:
@@ -273,16 +263,48 @@ def check_transform_on_handles(
                         check_row_order=check_row_order,
                     ):
                         raise ValueError(f"{db_handle} SQL result did not match expect")
-                if run_ops_version:
-                    if not equivalent_frames(
-                        res_db_ops,
-                        expect,
-                        float_tol=float_tol,
-                        check_column_order=check_column_order,
-                        cols_case_sensitive=cols_case_sensitive,
-                        check_row_order=check_row_order,
-                    ):
-                        raise ValueError(f"{db_handle} ops result did not match expect")
+                if not equivalent_frames(
+                    res_db_ops,
+                    expect,
+                    float_tol=float_tol,
+                    check_column_order=check_column_order,
+                    cols_case_sensitive=cols_case_sensitive,
+                    check_row_order=check_row_order,
+                ):
+                    raise ValueError(f"{db_handle} ops result did not match expect")
+
+
+def get_test_dbs():
+    """
+    handles connected to databases for testing.
+
+    """
+    db_handles = []
+    hdl = data_algebra.SQLite.example_handle()
+    assert hdl is not None
+    assert hdl.conn is not None
+    db_handles.append(hdl)
+    if test_PostgreSQL:
+        hdl = data_algebra.PostgreSQL.example_handle()
+        assert hdl is not None
+        assert hdl.conn is not None
+        db_handles.append(hdl)
+    if test_BigQuery:
+        hdl = data_algebra.BigQuery.example_handle()
+        assert hdl is not None
+        assert hdl.conn is not None
+        db_handles.append(hdl)
+    if test_MySQL:
+        hdl = data_algebra.MySQL.example_handle()
+        assert hdl is not None
+        assert hdl.conn is not None
+        db_handles.append(hdl)
+    if test_Spark:
+        hdl = data_algebra.SparkSQL.example_handle()
+        assert hdl is not None
+        assert hdl.conn is not None
+        db_handles.append(hdl)
+    return db_handles
 
 
 def check_transform(
@@ -328,29 +350,10 @@ def check_transform(
         data_algebra.PostgreSQL.PostgreSQLModel().db_handle(None),
         data_algebra.SparkSQL.SparkSQLModel().db_handle(None),
         data_algebra.MySQL.MySQLModel().db_handle(None),
-        data_algebra.SQLite.example_handle(),  # actual database instance, not empty
     ]
 
-    if test_PostgreSQL:
-        hdl = data_algebra.PostgreSQL.example_handle()
-        assert hdl is not None
-        assert hdl.conn is not None
-        db_handles.append(hdl)
-    if test_BigQuery:
-        hdl = data_algebra.BigQuery.example_handle()
-        assert hdl is not None
-        assert hdl.conn is not None
-        db_handles.append(hdl)
-    if test_MySQL:
-        hdl = data_algebra.MySQL.example_handle()
-        assert hdl is not None
-        assert hdl.conn is not None
-        db_handles.append(hdl)
-    if test_Spark:
-        hdl = data_algebra.SparkSQL.example_handle()
-        assert hdl is not None
-        assert hdl.conn is not None
-        db_handles.append(hdl)
+    test_dbs = get_test_dbs()
+    db_handles = db_handles + test_dbs
 
     if models_to_skip is not None:
         db_handles = [h for h in db_handles if str(h.db_model) not in models_to_skip]
